<?php
/*
  Plugin Name: Deluge Mail Forwarding Plugin
  Plugin URI: https://deluge.cc/?p=wordpress-plugin
  Description: A plugin that allows configuring email forwarding options through the deluge.cc routing service.
  Version: 0.4.1
  Author: mmeyer2k
  Author URI: https://github.com/mmeyer2k
  License: GPL2 or Later
 */

# hooks
add_action('admin_init', 'dcc_options_init');
add_action('admin_menu', 'dcc_options_add_page');

# api handler

function dcc_api_req($request) {
    if (ini_get('allow_url_fopen')):
        return file_get_contents($request);
    elseif (function_exists('curl_version')):
        $curl = curl_init($request);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        $data = curl_exec($curl);
        curl_close($curl);
        return $data;
    endif;
}

# whitelist options

function dcc_options_init() {
    register_setting('dcc_options_options', 'dcc_', 'dcc_options_validate');
}

# add menu page

function dcc_options_add_page() {
    add_options_page('Deluge Mail Options', 'Deluge Mail Options', 'manage_options', 'dcc_options', 'dcc_options_do_page');
}

# draw the menu page itself

function dcc_options_do_page() {
    $domain = $_SERVER['HTTP_HOST'];

    # declare a version variable, only needed in this scope
    $ver = '0.1';

    # parse domain to get only TLD
    if (substr_count($domain, '.')):
        $domain = explode('.', $domain);
        $domain = array_reverse($domain);
        $domain = "$domain[1].$domain[0]";
    endif;

    # some basic environment testing before we start firing off server requests
    if (!$domain)
        return;

    # gather option values
    $options = get_option('dcc_');
    $email = $options['email'];
    $token = $options['token'];
    $identifier = "&domain=$domain&token=$token&email=$email";

    # access the API and check if email/token are valid
    $verify = dcc_api_req("https://deluge.cc/?action=apiverify&email=$email&token=$token");
    $verify = $verify === '1';

    # set the api location
    $apiloc = 'https://deluge.cc';

    # use api to save settings if postback
    $msg = '';
    if (is_array($_POST) && isset($_POST['action'])):
        if ($_POST['action'] === 'apisavesinkhole'):
            $sinkhole = $_POST['sinkhole'];
            dcc_api_req("$apiloc?action=apisavesinkhole&sinkhole=$sinkhole" . $identifier);
            $msg = "Sinkhole address updated";
        elseif ($_POST['action'] === 'apisaveforward'):
            $target = $_POST['target'];
            $destination = $_POST['destination'];
            dcc_api_req("$apiloc?action=apisaveforward&target=$target&destination=$destination" . $identifier);
            $msg = "Forwarding address saved";
        elseif ($_POST['action'] === 'apideleteforward'):
            $target = $_POST['target'];
            dcc_api_req("$apiloc?action=apideleteforward&target=$target" . $identifier);
            $msg = "Forwarding address deleted";
        endif;
    endif;
    ?>
    <div class="wrap">
        <style>
            .dcc_input {
                padding: 7px;
                width: 300px;
                text-align: center;
                font-weight: bold;
            }
        </style>

        <h2>Deluge Mail Options</h2>
        <?php if ($msg): ?>
            <div style="text-align: center">
                <h1><?php echo $msg ?></h1>
            </div>
    <?php endif ?>

            <?php if (!$verify): ?>

            <form class="dcc_verify" method="post" action="options.php" style="text-align:center;">
        <?php settings_fields('dcc_options_options'); ?>
                <h3>
                    Please enter a valid API key and email address.
                </h3>
                <input class="dcc_input" name="dcc_[email]" placeholder="Registered e-mail address" type="email" value="<?php echo $options['email']; ?>"  /><br>
                <input class="dcc_input" name="dcc_[token]" placeholder="API Key" type="text" value="<?php echo $options['token']; ?>" /><br>
                <a href="https://deluge.cc/?p=register" target="_blank">Create account</a>  |  <a href="https://deluge.cc/?p=apikeys" target="_blank">Create API key</a>

                <p class="submit">
                    <input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
                </p>
        <?php if ($token && !$verify): ?>
                    <div style="border: 1px dotted red; padding: 10px;">
                        <h2>Your API key is not valid!</h3>
                            <ul>
                                <li>Make sure your server supports the file_get_contents() method.
                                <li>Ensure that you have a valid account at <a href="https://deluge.cc">deluge.cc</a>.
                                <li>Confirm that your API key is listed on your account <a href="">[link to api keys]</a>.
                            </ul>
                    </div>
            <?php endif ?>
            </form>

    <?php else: ?>

            <div style="
                 padding: 5px; 
                 text-align: center; 
                 border: 1px dotted green; 
                 margin-left: auto; 
                 margin-right: auto;">
                <strong>Deluge.cc API successfully configured.</strong>
                <form class="dcc_verify" method="post" action="options.php" style="display: inline;">
                    <button>Reset</button>
        <?php settings_fields('dcc_options_options'); ?>
                    <input class="dcc_input" name="dcc_[email]" placeholder="Registered e-mail address" type="hidden" value="" /><br>
                    <input class="dcc_input" name="dcc_[token]" placeholder="API Key" type="hidden" value="" />
                </form>
            </div>
            <br>

            <?php
            # check if domain is added for this user.
            $domain_active = dcc_api_req("https://deluge.cc?action=apidomaincheck&domain=$domain&token=$token");
            $domain_active = $domain_active === '1';

            if ($domain_active):
                # read the forwards for domain
                $forwards = dcc_api_req("https://deluge.cc?action=apireadforwards&email=$email&domain=$domain&token=$token");

                # explode forwards string into array
                $forwards = explode(',', $forwards);

                # read the sink hole for domain
                $sinkhole = dcc_api_req("https://deluge.cc?action=apireadsinkhole" . $identifier);
                ?>
                <div style="text-align: center;">
                    <h2>Mail forwarding options for <?php echo $domain; ?></h2>

                    <hr>
                    <h3>Custom Forwarding Addresses</h3>
                    <table style="margin-left: auto; margin-right: auto;">
                        <?php if ($forwards): ?>
                            <?php foreach ($forwards as $f): ?>

                                <?php
                                # if $f is nothing, do not create row
                                if (!$f)
                                    continue;

                                # break out contents of this forward
                                $x = explode(':', $f);
                                $targ = $x[0];
                                $dest = $x[1];
                                #echo var_dump($f);
                                ?>

                                <tr>
                                    <td><?php echo $targ ?>
                                    <td>@<?php echo $domain ?>
                                    <td style="font-weight: bold; font-size: 16pt;">&rArr;
                                    <td><?php echo $dest ?>
                                    <td>
                                        <form method="POST" action="">
                                            <input type="hidden" name="action" value="apideleteforward">
                                            <input type="hidden" name="d" value="<?php echo $domain ?>">
                                            <input type="hidden" name="target" value="<?php echo $targ ?>">
                                            <button class="button danger">Delete</button>
                                        </form>
                                </tr>
                <?php endforeach; ?>
            <?php endif; ?>
                        <form method="POST" action="">
                            <input type="hidden" name="action" value="apisaveforward">
                            <input type="hidden" name="domain" value="<?php echo $domain->domain ?>">
                            <tr>
                                <td><input type="text" name="target" style="width: 100px;">
                                <td>@<?php echo $domain ?>
                                <td style="font-weight: bold; font-size: 16pt;">&rArr;
                                <td><input placeholder="destination@address.com" type="email" name="destination" style="width: 200px;">
                                <td><button class="button">Add</button>
                            </tr>
                        </form>
                    </table>

                    <hr>
                    <h3>Sinkhole Address</h3>
                    <p>
                        A sinkhole address acts like a wildcard. Mail sent to any address at this domain <br>
                        will be forwarded to the sinkhole if no other forwards are matching.
                    </p>
                    <form action="" method="POST">
                        <input type="hidden" name="action" value="apisavesinkhole">
                        <input type="hidden" name="email" value="<?php echo $email; ?>">
                        <input type="hidden" name="domain" value="<?php echo $domain; ?>">
                        <input type="hidden" name="token" value="<?php echo $token; ?>">
                        <input type="email" name="sinkhole" value="<?php echo $sinkhole; ?>" class="dcc_input"><br>
                        <button class="button">Save</button>
                    </form>
                </div>
            <?php
        else:
            ?>
                <div style="padding: 10px; border: 1px dotted #c0c0c0; text-align: center;">
                    This domain <strong>[<?php echo $domain; ?>]</strong> has not been linked 
                    to your <a href="https://deluge.cc">deluge.cc</a> account. <a href="https://deluge.cc/?p=domain.new" target="_blank">Add domain now</a>.
                </div>
            <?php
            endif;
            ?>

    <?php endif ?>
    </div>
    <?php
}

# Sanitize and validate input. Accepts an array, return a sanitized array.

function dcc_options_validate($input) {
    # Our first value is either 0 or 1
    $input['option1'] = ( $input['option1'] == 1 ? 1 : 0 );

    # Say our second option must be safe text with no HTML tags
    $input['sometext'] = wp_filter_nohtml_kses($input['sometext']);

    return $input;
}
